char *boot[] = {
                "                    ",
                "   SHEDLANDIA.COM   ",
                " HYDROPONICS SYSTEM ",
                "                    "
                };

char *summary[] = {
                "  SUMMARY SETTINGS  ",
                " WATER: XX x XX MIN ",
                " LIGHT: XX HOURS    ",
                " HEAT:  XX C        "
                };

char *stats[] = {
                "   CURRENT STATUS   ",
                " WATER: OFF         ",
                " LIGHT: OFF         ",
                " HEAT:  OFF (XX C)  "
                };

char *water[] = {
                "   WATER SETTINGS   ",
                " FLOWS PER DAY: XX  ",
                " FLOW DURATION: XX  ",
                " FLOW RATE:     XXX%"
                };


char *light[] = {
                "   LIGHT SETTINGS   ",
                " HOURS PER DAY: XX  ",
                " INTENSITY:     XXX%",
                "                    "
                };
                
char *heat[] = {
                "   HEAT SETTINGS    ",
                " TARGET TEMP:   XX  ",
                " VARIABILITY:   XX  ",
                "                    "
                };

// The row column positions for tabs on each page
byte tabs[5][4][2]={
  {{1,8},{1,13},{2,8},{3,8}},
  {{1,8},{2,8},{3,8},{3,13}},
  {{1,16},{2,16},{3,16},{0,0}},
  {{1,16},{2,16},{0,0},{0,0}},
  {{1,16},{2,16},{0,0},{0,0}},
  };

  // EEPROM ADDRESS
// the addresses used for the values stored in eeprom
// plenty of capacity, so just assume longs
const byte ESTORE_waterFlows = 0;
const byte ESTORE_waterDuration = 4;
const byte ESTORE_waterPWM = 8; 
const byte ESTORE_lightDuration = 12; 
const byte ESTORE_lightPWM = 16; 
const byte ESTORE_tempDegC = 20; 
const byte ESTORE_tempRange = 24;

void show(char *page[]){
  // Clear LCD and display lines. 
  // Requires an array of char arrays
  // one line for each line on the display
  lcd.home();
  lcd.print(page[0]);
  lcd.print(page[2]);
  lcd.print(page[1]);
  lcd.print(page[3]);
    
  }

void pageBtnCheck(){
  // check for a page button
  // 0=summary 1=stats 2=water 3=light 4=heat
  // increment the page position
  if (digitalRead(btnPage) == LOW){
    // check for changes and save to EEPROM if necesssary
    if (ERQ == true){
      EEPROM.write(ESTORE_waterFlows, waterFlows);
      EEPROM.write(ESTORE_waterDuration, waterDuration);
      EEPROM.write(ESTORE_waterPWM, waterPWM); 
      EEPROM.write(ESTORE_lightDuration, lightDuration); 
      EEPROM.write(ESTORE_lightPWM, lightPWM); 
      EEPROM.write(ESTORE_tempDegC, tempDegC); 
      EEPROM.write(ESTORE_tempRange, tempRange);
      ERQ = false; // remove the request for update
      }    
    
    // move the currentPage marker to keep track of where we are
    if (currentPage == 4){
      currentPage = 0;
    } else {
      currentPage += 1;
    }  

    // page changed so request an update
    URQ = true;

    //update the display
    if (currentPage == 0) {
      show(summary); 
    }else if (currentPage == 1){
      show(stats);
    }else if (currentPage == 2){
      show(water);  
    }else if (currentPage == 3){  
      show(light);
    }else if (currentPage == 4){    
      show(heat);
    }
    
    //reset the tab position
    currentTab = 0;

  }
}

void tabBtnCheck(){
  // set the cursor positions of the tab points on each screen, into array of point
  // 0=summary 1=stats 2=water 3=light 4=heat
  // Don't forget the LCD row order is weird! 0,2,1,3
  byte numTabs=0;
  
  // Summary pages don't use tabs, so don't do anything on these pages
  if (currentPage == 0 or currentPage == 1){
    lcd.noBlink();
    return;
  } 

  // enumerate the number of tabstops for each page
  if (currentPage == 2){
    numTabs = 2;
  } else if (currentPage == 3 or currentPage == 4){
    numTabs = 1;
  }
  
  // move the cursor to the first position
  lcd.setCursor(tabs[currentPage][currentTab][1],tabs[currentPage][currentTab][0]);
  lcd.blink();

  if (digitalRead(btnTab) == LOW){
    // loop back to position 0 if we reach end 
    // remember sizeof returns bytesize, not elements
    if (currentTab >= numTabs){
      currentTab = 0;
    } else {
      currentTab += 1;
    }
  }
}

void upDownBtnCheck(){
  //updown doesn't do anything on these pages
  if (currentPage == 0 or currentPage == 1){
    return;
    }
    if (digitalRead(btnUp) == LOW){
      // we've pushed a button, so request an EEPROM save
      ERQ = true;

      if (currentPage == 2) {
        // water page
        if (currentTab == 0){
          if (waterFlows >= 23){waterFlows = 1;} else {waterFlows +=1;}          
          }
          
        if (currentTab == 1){
          if (waterDuration >= 60){ waterDuration = 0;} else {waterDuration +=1;}
          }
          
        if (currentTab == 2){
          if (waterPWM >= 100){ waterPWM = 0;} else {waterPWM +=5;}
          }
    
       
      } else if (currentPage == 3) {
        // light page
        if (currentTab == 0){
          if (lightDuration >= 23){lightDuration = 1;} else {lightDuration +=1;}
          }
        if (currentTab == 1){
          if (lightPWM >= 100){ lightPWM = 0;} else {lightPWM +=5;}
          }
    
      
      } else if (currentPage == 4) {
        // heat page
        if (currentTab == 0){
          if (tempDegC >= 40){tempDegC = 0;} else {tempDegC +=1;}
          }
        if (currentTab == 1){
          if (tempRange >= 10){ tempRange = 0;} else {tempRange +=1;}
          }
      }
  // request a screen update
  URQ = true;
  
  } else if(digitalRead(btnDown) == LOW){
     // we've pushed a button, so request an EEPROM save
     ERQ = true;

     if (currentPage == 2) {
        // water page
        if (currentTab == 0){
          if (waterFlows <= 0){waterFlows = 23;} else {waterFlows -=1;}          
          }
          
        if (currentTab == 1){
          if (waterDuration <= 0){ waterDuration = 60;} else {waterDuration -=1;}
          }
          
        if (currentTab == 2){
          if (waterPWM <= 0){ waterPWM = 100;} else {waterPWM -=5;}
          }
    
       
      } else if (currentPage == 3) {
        // light page
        if (currentTab == 0){
          if (lightDuration <= 0){lightDuration = 23;} else {lightDuration -=1;}
          }
        if (currentTab == 1){
          if (lightPWM <= 0){ lightPWM = 100;} else {lightPWM -=5;}
          }
    
      
      } else if (currentPage == 4) {
        // heat page
        if (currentTab == 0){
          if (tempDegC <= 0){tempDegC = 40;} else {tempDegC -=1;}
          }
        if (currentTab == 1){
          if (tempRange <= 0){ tempRange = 10;} else {tempRange -=1;}
          }
      }
  // request a screen update
  URQ = true;
  } 

}
void updateDisplayVals(){
  // exit if we don't need to update the values
  // live stats updates constantly, so except this
  if (URQ == false and currentPage != 1) {
    return;
    }

  //update the values on each page
  char val[3];
  if (currentPage == 0){
      // summary page
      lcd.setCursor(tabs[0][0][1],tabs[0][0][0]);
      sprintf(val, "%2d", waterFlows);
      lcd.print(val);

      lcd.setCursor(tabs[0][1][1],tabs[0][1][0]);
      sprintf(val, "%2d", waterDuration);
      lcd.print(val);
      
      lcd.setCursor(tabs[0][2][1],tabs[0][2][0]);
      sprintf(val, "%2d", lightDuration);
      lcd.print(val);
      
      lcd.setCursor(tabs[0][3][1],tabs[0][3][0]);
      sprintf(val, "%2d", tempDegC);
      lcd.print(val);
      
  } else if (currentPage == 1) {
      // live stats
      lcd.setCursor(tabs[1][0][1],tabs[1][0][0]);
      if (waterON){lcd.print("ON ");} else {lcd.print("OFF");};
      
      lcd.setCursor(tabs[1][1][1],tabs[1][1][0]);
      if (lightON){lcd.print("ON ");} else {lcd.print("OFF");};
      
      lcd.setCursor(tabs[1][2][1],tabs[1][2][0]);
      if (heatON){lcd.print("ON ");} else {lcd.print("OFF");};
       
      lcd.setCursor(tabs[1][3][1],tabs[1][3][0]);
      sprintf(val, "%2d", currentTemp);
      lcd.print(val);

      
  } else if (currentPage == 2) {
      // water page
      lcd.setCursor(tabs[2][0][1],tabs[2][0][0]);
      sprintf(val, "%3d", waterFlows);
      lcd.print(val);
      
      lcd.setCursor(tabs[2][1][1],tabs[2][1][0]);
      sprintf(val, "%3d", waterDuration);
      lcd.print(val);

      lcd.setCursor(tabs[2][2][1],tabs[2][2][0]);
      sprintf(val, "%3d", waterPWM);
      lcd.print(val);
    
  } else if (currentPage == 3) {
      // light page
      lcd.setCursor(tabs[3][0][1],tabs[3][0][0]);
      sprintf(val, "%3d", lightDuration);
      lcd.print(val);

      lcd.setCursor(tabs[3][1][1],tabs[3][1][0]);
      sprintf(val, "%3d", lightPWM);
      lcd.print(val);
    
  } else if (currentPage == 4) {
      // heat page
      lcd.setCursor(tabs[4][0][1],tabs[4][0][0]);
      sprintf(val, "%3d", tempDegC);
      lcd.print(val);

      lcd.setCursor(tabs[4][1][1],tabs[4][1][0]);
      sprintf(val, "%3d", tempRange);
      lcd.print(val);
  }
  // Update complete no more update required
  URQ = false; 
}

void doLoopTasks(){
  // do the checks that happen every loop - checking buttons and updating screens, etc...
  pageBtnCheck();
  updateDisplayVals();
  tabBtnCheck();
  upDownBtnCheck();
  }
